<?php
use Documents\Tag;
use Documents\Image;


class Image_Model_ImageModel extends Jme_Model
{
	protected $_sessionKey = 'Image_ImageModel_SESSION';
	
	/**
	 * @var array
	 */
	private $_forms;
	
	/**
	 * @var Doctrine\ODM\MongoDB\DocumentManager
	 */
	private $_dm;
	
	/**
	 * @var string
	 */
	private $_publicDir;
	
	/**
	 * Available forms:
	 * 
	 * - image
	 * - description
	 * - tags
	 * 
	 * @param string $name form name
	 * @return Zend_Form
	 */
	public function getForm($name = 'image')
	{
		$form = null;
		
		if(!isset($this->_forms[$name]) )
		{
			switch($name)
			{
				case 'image':
				{
					$form = new Image_Form_AddImageForm();
					$form->setAction('/image/image/addimage');
					
					break;
				}
				case 'description':
				{
					$form = new Image_Form_AddDescriptionForm();
					$form->setAction('/image/image/adddescription');
					
					break;
				}
				case 'tags':
				{
					$form = new Image_Form_AddTagsForm();
					$form->setAction('/image/image/addtags');
					break;
				}	
				case 'search':
				{
					$form = new Image_Form_SearchImagesForm();
					$form->setAction('/image/image/search');
					
					break;
				}					
				default:
				{
					throw new Jme_Form_IllegalArgumentException('Invalid form name: ' . $name, 500);
				}							
			}	
			
			$this->_forms[$name] = $form;
		}			
		
		return $this->_forms[$name];
	}
	
	/**
	 * @param Doctrine\ODM\MongoDB\DocumentManager $dm
	 */
	public function setDocumentManager(Doctrine\ODM\MongoDB\DocumentManager $dm)
	{
		$this->_dm = $dm;
	}

	public function setUploadDirectory($path)
	{
		$this->getForm()->getElement('image')->setDestination($path);
	}
	
	public function setPublicDirectory($path)
	{
		$this->_publicDir = $path;
	}
	
	public function addImage(array $data)
	{
		$form = $this->getForm('image');
		
	    if($form->isValid($data) )
	    {
	    	$adapter = $form->getElement('image')->getTransferAdapter();

	    	$info = $adapter->getFileInfo();
	    	
	    	$path = $this->getForm()->getElement('image')->getDestination() . DIRECTORY_SEPARATOR . $info['image']['name'];	    	
	    	
	    	$uniqueName = $this->_createUniqueFilename($path);
	    	
	    	$adapter->addFilter('rename', array('target' => $uniqueName, 'overwrite' => true) );
	    	
	    	if(!$adapter->receive() )
	    	{
	    		die('error');
	    	}
	    	
	    	$image = new Image();
	    	$image->setName($info['image']['name']);
	    	$image->setFile($uniqueName);
	    	$image->setPublicDir($this->_publicDir);
	    	
	    	$this->_dm->persist($image);
	    	$this->_dm->flush();
	    	
	    	$ns = $this->getSession();
	    	$ns->imageId = $image->getId();
	    	
	    	return true;
	    }
	    
	    return false;
	}
	
	protected function _createUniqueFilename($filename)
	{
		$info = pathinfo($filename);
		
		$dirname = $info['dirname'];
		$extension = $info['extension'];
		$filename = $info['filename'];
		
		do 
		{
			$generated = $dirname . DIRECTORY_SEPARATOR . md5($filename . uniqid() . rand() ) . ".$extension";
		}
		while(file_exists($generated) );
		
		return $generated;
	}
	
	
	public function updateImage(array $data)
	{
		$form = $this->getForm('description');
		
	    if($form->isValid($data) )	
	    {	
			$ns = $this->getSession();
			
			$image = $this->_dm->createQuery('\Documents\Image')->field('id')->equals($ns->imageId)->getSingleResult();
			
			return true;
	    }
		
	    return false;
	}
	
	public function updateDescription(array $data)
	{
		$form = $this->getForm('description');
		
	    if($form->isValid($data) )	
	    {			
			$ns = $this->getSession();
			
			$image = $this->_dm->createQuery('\Documents\Image')->field('id')->equals($ns->imageId)->getSingleResult();
			$image->setDescription($data['description']);
			
	    	$this->_dm->persist($image);
	    	$this->_dm->flush();
	    	
	    	$img = $this->getImage($ns->imageId);
	    	
	    	return true;
	    }
	    
	    return false;
	}
	
	public function updateTags(array $data)
	{
		$form = $this->getForm('tags');
		
	    if($form->isValid($data) )	
	    {			
			$ns = $this->getSession();
			
			$image = $this->_dm->createQuery('\Documents\Image')->field('id')->equals($ns->imageId)->getSingleResult();
			
			foreach($data['tags'] as $tagName)
			{
				if(strlen(trim($tagName) ) > 0)
				{
					$image->addTag($tagName);
				}
			}
			
	    	$this->_dm->persist($image);
	    	$this->_dm->flush();		
	    	
	    	return true;
	    }
	    
	    return false;
	}
	
	public function getCurrentImage()
	{
		$ns = $this->getSession();
		
		return $this->getImage($ns->imageId);
	}
	
	public function getImage($id)
	{
		$image = $this->_dm->createQuery('\Documents\Image')->field('id')->equals($id)->getSingleResult();
		
		return $image;
	}
	
	public function search(array $params = array() )
	{
		$query = $this->_dm->createQuery('\Documents\Image');
		
		if(isset($params['tags']) )
		{
			$tagsArr = explode(',', $params['tags']);
			
			$tags = array();
			
			foreach($tagsArr as $el)
			{
				$el = trim($el);
				
				if(strlen($el) != 0)
				{
					$tags[] = $el;
				}
			}
			
			return $query->field('tags')->in($tags)->execute();
		}
	}
	
	public function processStarted()
	{
		return isset($this->getSession()->started) && $this->getSession()->started === true;
	}
	
	/**
	 * @param boolean $force if true then a new process is started regardless if an existing and valid process exists
	 */
	public function startProcess($force = true)
	{
		$ns = $this->getSession();
		
		if($force || !$this->processStarted() )
		{
			// init new process
			$ns = $this->initSession();
			$ns->started = true;
		}
		
		$this->_ns = $ns;
	}
	
	public function endProcess()
	{
		$ns = $this->initSession();
		$ns->started = false;
	}
}